/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.service.types;

import static java.util.Collections.emptyList;
import static java.util.Collections.unmodifiableCollection;

import java.util.Collection;
import java.util.Set;

/** Maps a given EMF type to a {@link Set} of types. */
public class EMFTypeToTypeSetMap<S, T> extends EMFTypeMap<S, TypeSet<T>> {

	/** Adds the given {@code targetType} to the mapped types of the given {@code sourceType}. */
	public void addTypeSetEntry(Class<? extends S> sourceType, Class<? extends T> targetType) {

		TypeSet<T> typeSet = get(sourceType);
		if(typeSet == null) {
			typeSet = new TypeSet<T>();
			put(sourceType, typeSet);
		}
		typeSet.add(targetType);
	}

	/**
	 * Returns the (possibly empty) unmodifiable {@link Collection} of target types that are mapped
	 * to the given given {@code sourceType}.
	 */
	public Collection<Class<? extends T>> getTargetTypes(Class<? extends S> sourceType) {
		final TypeSet<T> rval = get(sourceType);
		return rval != null ? unmodifiableCollection(rval) : emptyList();
	}
}
